@@ -31,7 +31,8 @@
 #include "arrow/compute/kernel.h"
 #include "arrow/compute/kernels/codegen_internal.h"
 #include "arrow/compute/registry.h"
-#include "arrow/util/logging.h"
+#include "arrow/compute/registry_internal.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/reflection_internal.h"
 
 namespace arrow {
