@@ -17,8 +17,11 @@
 
 #include "benchmark/benchmark.h"
 
+#include "arrow/array.h"
+#include "arrow/chunked_array.h"
 #include "arrow/compute/api_vector.h"
-#include "arrow/compute/kernels/test_util.h"
+#include "arrow/compute/exec.h"
+#include "arrow/datum.h"
 #include "arrow/table.h"
 #include "arrow/testing/gtest_util.h"
 #include "arrow/testing/random.h"
@@ -27,7 +30,9 @@
 
 namespace arrow {
 namespace compute {
+
 constexpr auto kSeed = 0x0ff1ce;
+constexpr int32_t kDictionarySize = 24;  // a typical dictionary size
 
 //
 // Array sort/rank benchmark helpers
@@ -70,6 +75,22 @@ static void ArraySortFuncBenchmark(benchmark::State& state, const Runner& runner
   state.SetItemsProcessed(state.iterations() * values->length());
 }
 
+// Compute the number of values in a StringArray based on a target `num_bytes`
+static int64_t GetStringArraySize(int64_t num_bytes, int32_t min_length,
+                                  int32_t max_length, double null_proportion,
+                                  int num_chunks = 1) {
+  // The array_size is derived from these two equations:
+  //
+  //     num_bytes = array_size * (1.0 - null_proportion) * string_mean_length
+  //     string_mean_length = (max_length + min_length) / 2.0
+  const double valid_proportion = 1.0 - null_proportion;
+  const double array_size =
+      (valid_proportion > std::numeric_limits<double>::epsilon())
+          ? (num_bytes * 2) / (valid_proportion * (max_length + min_length))
+          : (num_bytes * 2) / (max_length + min_length);
+  return static_cast<int64_t>(std::round(array_size / num_chunks));
+}
+
 template <typename Runner>
 static void ArraySortFuncInt64Benchmark(benchmark::State& state, const Runner& runner,
                                         int64_t min, int64_t max) {
@@ -82,6 +103,78 @@ static void ArraySortFuncInt64Benchmark(benchmark::State& state, const Runner& r
   ArraySortFuncBenchmark(state, runner, values);
 }
 
+template <typename Runner>
+static void ArraySortFuncInt64DictBenchmark(benchmark::State& state, const Runner& runner,
+                                            int64_t min, int64_t max, int32_t dict_size) {
+  RegressionArgs args(state);
+
+  double null_proportion = args.null_proportion;
+  double values_null_proportion = null_proportion / 2;
+  if (1.0 - null_proportion < std::numeric_limits<double>::epsilon()) {
+    values_null_proportion = null_proportion = 1.0;
+  }
+  double indices_null_proportion = null_proportion - values_null_proportion;
+
+  const int64_t array_size = args.size / sizeof(int64_t);
+
+  auto rand = random::RandomArrayGenerator(kSeed);
+  auto dict_values = rand.Int64(dict_size, min, max, values_null_proportion);
+  auto dict_indices = rand.Int64(array_size, 0, dict_size - 1, indices_null_proportion);
+  auto dict_array = *DictionaryArray::FromArrays(dict_indices, dict_values);
+
+  ArraySortFuncBenchmark(state, runner, dict_array);
+}
+
+template <typename Runner>
+static void ArraySortFuncStringBenchmark(benchmark::State& state, const Runner& runner,
+                                         int32_t min_length, int32_t max_length) {
+  RegressionArgs args(state);
+
+  const int64_t array_size =
+      GetStringArraySize(args.size, min_length, max_length, args.null_proportion);
+
+  auto rand = random::RandomArrayGenerator(kSeed);
+  auto values = rand.String(array_size, min_length, max_length, args.null_proportion);
+
+  ArraySortFuncBenchmark(state, runner, values);
+}
+
+template <typename Runner>
+static void ArraySortFuncStringDictBenchmark(benchmark::State& state,
+                                             const Runner& runner, int32_t min_length,
+                                             int32_t max_length, int32_t dict_size) {
+  RegressionArgs args(state);
+
+  double null_proportion = args.null_proportion;
+  double values_null_proportion = null_proportion / 2;
+  if (1.0 - null_proportion < std::numeric_limits<double>::epsilon()) {
+    values_null_proportion = null_proportion = 1.0;
+  }
+  double indices_null_proportion = null_proportion - values_null_proportion;
+
+  const int64_t array_size =
+      GetStringArraySize(args.size, min_length, max_length, null_proportion);
+
+  auto rand = random::RandomArrayGenerator(kSeed);
+  auto dict_values =
+      rand.String(dict_size, min_length, max_length, values_null_proportion);
+  auto dict_indices = rand.Int64(array_size, 0, dict_size - 1, indices_null_proportion);
+  auto dict_array = *DictionaryArray::FromArrays(dict_indices, dict_values);
+
+  ArraySortFuncBenchmark(state, runner, dict_array);
+}
+
+template <typename Runner>
+static void ArraySortFuncBoolBenchmark(benchmark::State& state, const Runner& runner) {
+  RegressionArgs args(state);
+
+  const int64_t array_size = args.size * 8;
+  auto rand = random::RandomArrayGenerator(kSeed);
+  auto values = rand.Boolean(array_size, 0.5, args.null_proportion);
+
+  ArraySortFuncBenchmark(state, runner, values);
+}
+
 template <typename Runner>
 static void ChunkedArraySortFuncInt64Benchmark(benchmark::State& state,
                                                const Runner& runner, int64_t min,
@@ -100,14 +193,24 @@ static void ChunkedArraySortFuncInt64Benchmark(benchmark::State& state,
 }
 
 template <typename Runner>
-static void ArraySortFuncBoolBenchmark(benchmark::State& state, const Runner& runner) {
+static void ChunkedArraySortFuncStringBenchmark(benchmark::State& state,
+                                                const Runner& runner, int32_t min_length,
+                                                int32_t max_length) {
   RegressionArgs args(state);
 
-  const int64_t array_size = args.size * 8;
+  const auto n_chunks = 10;
+  const auto array_chunk_size = GetStringArraySize(args.size, min_length, max_length,
+                                                   args.null_proportion, n_chunks);
   auto rand = random::RandomArrayGenerator(kSeed);
-  auto values = rand.Boolean(array_size, 0.5, args.null_proportion);
 
-  ArraySortFuncBenchmark(state, runner, values);
+  ArrayVector chunks;
+  for (auto i = 0; i < n_chunks; ++i) {
+    auto values =
+        rand.String(array_chunk_size, min_length, max_length, args.null_proportion);
+    chunks.push_back(std::move(values));
+  }
+
+  ArraySortFuncBenchmark(state, runner, std::make_shared<ChunkedArray>(chunks));
 }
 
 static void ArraySortIndicesInt64Narrow(benchmark::State& state) {
@@ -128,6 +231,13 @@ static void ArraySortIndicesInt64Wide(benchmark::State& state) {
   ArraySortFuncInt64Benchmark(state, SortRunner(state), min, max);
 }
 
+static void ArraySortIndicesInt64WideDict(benchmark::State& state) {
+  const auto dict_size = kDictionarySize;
+  const auto min = std::numeric_limits<int64_t>::min();
+  const auto max = std::numeric_limits<int64_t>::max();
+  ArraySortFuncInt64DictBenchmark(state, SortRunner(state), min, max, dict_size);
+}
+
 static void ArrayRankInt64Wide(benchmark::State& state) {
   const auto min = std::numeric_limits<int64_t>::min();
   const auto max = std::numeric_limits<int64_t>::max();
@@ -144,6 +254,38 @@ static void ArraySortIndicesBool(benchmark::State& state) {
   ArraySortFuncBoolBenchmark(state, SortRunner(state));
 }
 
+static void ArraySortIndicesStringNarrow(benchmark::State& state) {
+  const auto min_length = 0;
+  const auto max_length = 16;
+  ArraySortFuncStringBenchmark(state, SortRunner(state), min_length, max_length);
+}
+
+static void ArraySortIndicesStringWide(benchmark::State& state) {
+  const auto min_length = 0;
+  const auto max_length = 64;
+  ArraySortFuncStringBenchmark(state, SortRunner(state), min_length, max_length);
+}
+
+static void ArraySortIndicesStringWideDict(benchmark::State& state) {
+  const auto dict_size = kDictionarySize;
+  const auto min_length = 0;
+  const auto max_length = 64;
+  ArraySortFuncStringDictBenchmark(state, SortRunner(state), min_length, max_length,
+                                   dict_size);
+}
+
+static void ArrayRankStringNarrow(benchmark::State& state) {
+  const auto min_length = 0;
+  const auto max_length = 16;
+  ArraySortFuncStringBenchmark(state, RankRunner(state), min_length, max_length);
+}
+
+static void ArrayRankStringWide(benchmark::State& state) {
+  const auto min_length = 0;
+  const auto max_length = 64;
+  ArraySortFuncStringBenchmark(state, RankRunner(state), min_length, max_length);
+}
+
 static void ChunkedArraySortIndicesInt64Narrow(benchmark::State& state) {
   ChunkedArraySortFuncInt64Benchmark(state, SortRunner(state), -100, 100);
 }
@@ -154,6 +296,12 @@ static void ChunkedArraySortIndicesInt64Wide(benchmark::State& state) {
   ChunkedArraySortFuncInt64Benchmark(state, SortRunner(state), min, max);
 }
 
+static void ChunkedArraySortIndicesString(benchmark::State& state) {
+  const auto min_length = 0;
+  const auto max_length = 32;
+  ChunkedArraySortFuncStringBenchmark(state, SortRunner(state), min_length, max_length);
+}
+
 //
 // Record batch and table sort benchmark helpers
 //
@@ -161,7 +309,7 @@ static void ChunkedArraySortIndicesInt64Wide(benchmark::State& state) {
 static void DatumSortIndicesBenchmark(benchmark::State& state, const Datum& datum,
                                       const SortOptions& options) {
   for (auto _ : state) {
-    ABORT_NOT_OK(SortIndices(datum, options).status());
+    ABORT_NOT_OK(SortIndices(datum, options));
   }
 }
 
@@ -308,10 +456,15 @@ void ArraySortIndicesSetArgs(benchmark::internal::Benchmark* bench) {
 
 BENCHMARK(ArraySortIndicesInt64Narrow)->Apply(ArraySortIndicesSetArgs);
 BENCHMARK(ArraySortIndicesInt64Wide)->Apply(ArraySortIndicesSetArgs);
+BENCHMARK(ArraySortIndicesInt64WideDict)->Apply(ArraySortIndicesSetArgs);
 BENCHMARK(ArraySortIndicesBool)->Apply(ArraySortIndicesSetArgs);
+BENCHMARK(ArraySortIndicesStringNarrow)->Apply(ArraySortIndicesSetArgs);
+BENCHMARK(ArraySortIndicesStringWide)->Apply(ArraySortIndicesSetArgs);
+BENCHMARK(ArraySortIndicesStringWideDict)->Apply(ArraySortIndicesSetArgs);
 
 BENCHMARK(ChunkedArraySortIndicesInt64Narrow)->Apply(ArraySortIndicesSetArgs);
 BENCHMARK(ChunkedArraySortIndicesInt64Wide)->Apply(ArraySortIndicesSetArgs);
+BENCHMARK(ChunkedArraySortIndicesString)->Apply(ArraySortIndicesSetArgs);
 
 BENCHMARK(RecordBatchSortIndicesInt64Narrow)
     ->ArgsProduct({
@@ -379,6 +532,8 @@ void ArrayRankSetArgs(benchmark::internal::Benchmark* bench) {
 
 BENCHMARK(ArrayRankInt64Narrow)->Apply(ArrayRankSetArgs);
 BENCHMARK(ArrayRankInt64Wide)->Apply(ArrayRankSetArgs);
+BENCHMARK(ArrayRankStringNarrow)->Apply(ArrayRankSetArgs);
+BENCHMARK(ArrayRankStringWide)->Apply(ArrayRankSetArgs);
 BENCHMARK(ChunkedArrayRankInt64Narrow)->Apply(ArrayRankSetArgs);
 BENCHMARK(ChunkedArrayRankInt64Wide)->Apply(ArrayRankSetArgs);
 
