@@ -35,6 +35,7 @@ int SwissTable::early_filter_imp_avx2_x8(const int num_hashes, const uint32_t* h
   constexpr int unroll = 8;
 
   const int num_group_id_bits = num_groupid_bits_from_log_blocks(log_blocks_);
+  const int num_block_bytes = num_block_bytes_from_num_groupid_bits(num_group_id_bits);
   const __m256i* vhash_ptr = reinterpret_cast<const __m256i*>(hashes);
   const __m256i vstamp_mask = _mm256_set1_epi32((1 << bits_stamp_) - 1);
 
@@ -53,7 +54,7 @@ int SwissTable::early_filter_imp_avx2_x8(const int num_hashes, const uint32_t* h
     // in order to process 64-bit blocks
     //
     __m256i vblock_offset =
-        _mm256_mullo_epi32(vblock_id, _mm256_set1_epi32(num_group_id_bits + 8));
+        _mm256_mullo_epi32(vblock_id, _mm256_set1_epi32(num_block_bytes));
     __m256i voffset_A = _mm256_and_si256(vblock_offset, _mm256_set1_epi64x(0xffffffff));
     __m256i vstamp_A = _mm256_and_si256(vstamp, _mm256_set1_epi64x(0xffffffff));
     __m256i voffset_B = _mm256_srli_epi64(vblock_offset, 32);
@@ -230,9 +231,10 @@ int SwissTable::early_filter_imp_avx2_x32(const int num_hashes, const uint32_t*
   // Assemble the sequence of block bytes.
   uint64_t block_bytes[16];
   const int num_groupid_bits = num_groupid_bits_from_log_blocks(log_blocks_);
+  const int num_block_bytes = num_block_bytes_from_num_groupid_bits(num_groupid_bits);
   for (int i = 0; i < (1 << log_blocks_); ++i) {
     uint64_t in_blockbytes =
-        *reinterpret_cast<const uint64_t*>(blocks_->data() + (8 + num_groupid_bits) * i);
+        *reinterpret_cast<const uint64_t*>(block_data(i, num_block_bytes));
     block_bytes[i] = in_blockbytes;
   }
 
@@ -365,14 +367,9 @@ int SwissTable::early_filter_imp_avx2_x32(const int num_hashes, const uint32_t*
 
 int SwissTable::extract_group_ids_avx2(const int num_keys, const uint32_t* hashes,
                                        const uint8_t* local_slots,
-                                       uint32_t* out_group_ids, int byte_offset,
-                                       int byte_multiplier, int byte_size) const {
-  ARROW_DCHECK(byte_size == 1 || byte_size == 2 || byte_size == 4);
-  uint32_t mask = byte_size == 1 ? 0xFF : byte_size == 2 ? 0xFFFF : 0xFFFFFFFF;
-  auto elements = reinterpret_cast<const int*>(blocks_->data() + byte_offset);
+                                       uint32_t* out_group_ids) const {
   constexpr int unroll = 8;
   if (log_blocks_ == 0) {
-    ARROW_DCHECK(byte_size == 1 && byte_offset == 8 && byte_multiplier == 16);
     __m256i block_group_ids =
         _mm256_set1_epi64x(reinterpret_cast<const uint64_t*>(blocks_->data())[1]);
     for (int i = 0; i < num_keys / unroll; ++i) {
@@ -385,33 +382,52 @@ int SwissTable::extract_group_ids_avx2(const int num_keys, const uint32_t* hashe
       _mm256_storeu_si256(reinterpret_cast<__m256i*>(out_group_ids) + i, group_id);
     }
   } else {
+    int num_groupid_bits = num_groupid_bits_from_log_blocks(log_blocks_);
+    int num_groupid_bytes = num_groupid_bits / 8;
+    uint32_t mask = num_groupid_bytes == 1   ? 0xFF
+                    : num_groupid_bytes == 2 ? 0xFFFF
+                                             : 0xFFFFFFFF;
+    int num_block_bytes = num_block_bytes_from_num_groupid_bits(num_groupid_bits);
+    const int* slots_base =
+        reinterpret_cast<const int*>(blocks_->data() + bytes_status_in_block_);
+
     for (int i = 0; i < num_keys / unroll; ++i) {
       __m256i hash = _mm256_loadu_si256(reinterpret_cast<const __m256i*>(hashes) + i);
-      // Extend hash and local_slot to 64-bit to compute 64-bit group id offsets to
-      // gather from. This is to prevent index overflow issues in GH-44513.
-      // NB: Use zero-extend conversion for unsigned hash.
-      __m256i hash_lo = _mm256_cvtepu32_epi64(_mm256_castsi256_si128(hash));
-      __m256i hash_hi = _mm256_cvtepu32_epi64(_mm256_extracti128_si256(hash, 1));
+      __m256i block_id =
+          _mm256_srlv_epi32(hash, _mm256_set1_epi32(bits_hash_ - log_blocks_));
+
       __m256i local_slot =
           _mm256_set1_epi64x(reinterpret_cast<const uint64_t*>(local_slots)[i]);
+
+      // Extend block_id and local_slot to 64-bit to compute 64-bit group id offsets to
+      // gather from. This is to prevent index overflow issues in GH-44513.
       __m256i local_slot_lo = _mm256_shuffle_epi8(
           local_slot, _mm256_setr_epi32(0x80808000, 0x80808080, 0x80808001, 0x80808080,
                                         0x80808002, 0x80808080, 0x80808003, 0x80808080));
       __m256i local_slot_hi = _mm256_shuffle_epi8(
           local_slot, _mm256_setr_epi32(0x80808004, 0x80808080, 0x80808005, 0x80808080,
                                         0x80808006, 0x80808080, 0x80808007, 0x80808080));
-      local_slot_lo = _mm256_mul_epu32(local_slot_lo, _mm256_set1_epi32(byte_size));
-      local_slot_hi = _mm256_mul_epu32(local_slot_hi, _mm256_set1_epi32(byte_size));
-      __m256i pos_lo = _mm256_srli_epi64(hash_lo, bits_hash_ - log_blocks_);
-      __m256i pos_hi = _mm256_srli_epi64(hash_hi, bits_hash_ - log_blocks_);
-      pos_lo = _mm256_mul_epu32(pos_lo, _mm256_set1_epi32(byte_multiplier));
-      pos_hi = _mm256_mul_epu32(pos_hi, _mm256_set1_epi32(byte_multiplier));
-      pos_lo = _mm256_add_epi64(pos_lo, local_slot_lo);
-      pos_hi = _mm256_add_epi64(pos_hi, local_slot_hi);
-      __m128i group_id_lo = _mm256_i64gather_epi32(elements, pos_lo, 1);
-      __m128i group_id_hi = _mm256_i64gather_epi32(elements, pos_hi, 1);
+      local_slot_lo =
+          _mm256_mul_epu32(local_slot_lo, _mm256_set1_epi32(num_groupid_bytes));
+      local_slot_hi =
+          _mm256_mul_epu32(local_slot_hi, _mm256_set1_epi32(num_groupid_bytes));
+
+      // NB: Use zero-extend conversion for unsigned block_id.
+      __m256i slot_offset_lo = _mm256_cvtepu32_epi64(_mm256_castsi256_si128(block_id));
+      __m256i slot_offset_hi =
+          _mm256_cvtepu32_epi64(_mm256_extracti128_si256(block_id, 1));
+      slot_offset_lo =
+          _mm256_mul_epi32(slot_offset_lo, _mm256_set1_epi64x(num_block_bytes));
+      slot_offset_hi =
+          _mm256_mul_epi32(slot_offset_hi, _mm256_set1_epi64x(num_block_bytes));
+      slot_offset_lo = _mm256_add_epi64(slot_offset_lo, local_slot_lo);
+      slot_offset_hi = _mm256_add_epi64(slot_offset_hi, local_slot_hi);
+
+      __m128i group_id_lo = _mm256_i64gather_epi32(slots_base, slot_offset_lo, 1);
+      __m128i group_id_hi = _mm256_i64gather_epi32(slots_base, slot_offset_hi, 1);
       __m256i group_id = _mm256_set_m128i(group_id_hi, group_id_lo);
       group_id = _mm256_and_si256(group_id, _mm256_set1_epi32(mask));
+
       _mm256_storeu_si256(reinterpret_cast<__m256i*>(out_group_ids) + i, group_id);
     }
   }
