@@ -27,17 +27,15 @@
 #include "parquet/encryption/file_system_key_material_store.h"
 #include "parquet/encryption/key_toolkit_internal.h"
 
-namespace parquet {
-namespace encryption {
+namespace parquet::encryption {
 
 std::shared_ptr<KmsClient> KeyToolkit::GetKmsClient(
     const KmsConnectionConfig& kms_connection_config, double cache_entry_lifetime_ms) {
-  if (kms_client_factory_ == NULL) {
+  if (kms_client_factory_ == nullptr) {
     throw ParquetException("No KmsClientFactory is registered.");
   }
-  auto kms_client_per_kms_instance_cache =
-      kms_client_cache_per_token().GetOrCreateInternalCache(
-          kms_connection_config.key_access_token(), cache_entry_lifetime_ms);
+  auto kms_client_per_kms_instance_cache = kms_client_cache_.GetOrCreateInternalCache(
+      kms_connection_config.key_access_token(), cache_entry_lifetime_ms);
 
   return kms_client_per_kms_instance_cache->GetOrInsert(
       kms_connection_config.kms_instance_id, [this, kms_connection_config]() {
@@ -89,7 +87,7 @@ void KeyToolkit::RotateMasterKeys(
                                             std::string(KeyMaterial::kFooterKeyIdInFile));
 
   // Rotate column keys
-  for (auto const& key_id_in_file : file_key_id_set) {
+  for (const auto& key_id_in_file : file_key_id_set) {
     if (key_id_in_file == std::string(KeyMaterial::kFooterKeyIdInFile)) {
       continue;
     }
@@ -119,5 +117,4 @@ void KeyToolkit::RemoveCacheEntriesForAllTokens() {
   kek_read_cache_per_token().Clear();
 }
 
-}  // namespace encryption
-}  // namespace parquet
+}  // namespace parquet::encryption
