@@ -129,20 +129,26 @@ StreamWriter& StreamWriter::operator<<(FixedStringView v) {
 }
 
 StreamWriter& StreamWriter::operator<<(const char* v) {
-  return WriteVariableLength(v, std::strlen(v));
+  return WriteVariableLength(v, std::strlen(v), ConvertedType::UTF8);
 }
 
 StreamWriter& StreamWriter::operator<<(const std::string& v) {
-  return WriteVariableLength(v.data(), v.size());
+  return WriteVariableLength(v.data(), v.size(), ConvertedType::UTF8);
 }
 
 StreamWriter& StreamWriter::operator<<(::std::string_view v) {
-  return WriteVariableLength(v.data(), v.size());
+  return WriteVariableLength(v.data(), v.size(), ConvertedType::UTF8);
+}
+
+StreamWriter& StreamWriter::operator<<(RawDataView v) {
+  return WriteVariableLength(reinterpret_cast<const char*>(v.data()), v.size(),
+                             ConvertedType::NONE);
 }
 
 StreamWriter& StreamWriter::WriteVariableLength(const char* data_ptr,
-                                                std::size_t data_len) {
-  CheckColumn(Type::BYTE_ARRAY, ConvertedType::UTF8);
+                                                std::size_t data_len,
+                                                ConvertedType::type type) {
+  CheckColumn(Type::BYTE_ARRAY, type);
 
   auto writer = static_cast<ByteArrayWriter*>(row_group_writer_->column(column_index_++));
 
@@ -157,7 +163,7 @@ StreamWriter& StreamWriter::WriteVariableLength(const char* data_ptr,
     writer->WriteBatch(kBatchSizeOne, &kDefLevelZero, &kRepLevelZero, nullptr);
   }
   if (max_row_group_size_ > 0) {
-    row_group_size_ += writer->EstimatedBufferedValueBytes();
+    row_group_size_ += writer->estimated_buffered_value_bytes();
   }
   return *this;
 }
@@ -178,7 +184,7 @@ StreamWriter& StreamWriter::WriteFixedLength(const char* data_ptr, std::size_t d
     writer->WriteBatch(kBatchSizeOne, &kDefLevelZero, &kRepLevelZero, nullptr);
   }
   if (max_row_group_size_ > 0) {
-    row_group_size_ += writer->EstimatedBufferedValueBytes();
+    row_group_size_ += writer->estimated_buffered_value_bytes();
   }
   return *this;
 }
